@@ -11,22 +11,19 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 /* "groupsock" interface
- * Copyright (c) 1996-2003 Live Networks, Inc.  All rights reserved.
+ * Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
  * Common include files, typically used for networking
  */
 
 #ifndef _NET_COMMON_H
 #define _NET_COMMON_H
 
-#if defined(IMN_PIM)
-#include "IMN_PIMNetCommon.h"
-
-#elif defined(__WIN32__) || defined(_WIN32) || defined(_WIN32_WCE)
+#if defined(__WIN32__) || defined(_WIN32) || defined(_WIN32_WCE)
 /* Windows */
-#if defined(WINNT) || defined(_WINNT) || defined(__BORLANDC__) || defined(__MINGW32__) || defined(_WIN32_WCE)
+#if defined(WINNT) || defined(_WINNT) || defined(__BORLANDC__) || defined(__MINGW32__) || defined(_WIN32_WCE) || defined (_MSC_VER)
 #define _MSWSOCK_
 #include <winsock2.h>
 #include <ws2tcpip.h>
@@ -34,17 +31,30 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include <windows.h>
 #include <string.h>
 
-#define _close closesocket
+#define closeSocket closesocket
 #define EWOULDBLOCK WSAEWOULDBLOCK
+#define EINPROGRESS WSAEWOULDBLOCK
+#define EAGAIN WSAEWOULDBLOCK
+#define EINTR WSAEINTR
 
 #if defined(_WIN32_WCE)
 #define NO_STRSTREAM 1
 #endif
 
 /* Definitions of size-specific types: */
+typedef __int64 int64_t;
+typedef unsigned __int64 u_int64_t;
 typedef unsigned u_int32_t;
 typedef unsigned short u_int16_t;
 typedef unsigned char u_int8_t;
+// For "uintptr_t" and "intptr_t", we assume that if they're not already defined, then this must be
+// an old, 32-bit version of Windows:
+#if !defined(_MSC_STDINT_H_) && !defined(_UINTPTR_T_DEFINED) && !defined(_UINTPTR_T_DECLARED) && !defined(_UINTPTR_T)
+typedef unsigned uintptr_t;
+#endif
+#if !defined(_MSC_STDINT_H_) && !defined(_INTPTR_T_DEFINED) && !defined(_INTPTR_T_DECLARED) && !defined(_INTPTR_T)
+typedef int intptr_t;
+#endif
 
 #elif defined(VXWORKS)
 /* VxWorks */
@@ -74,14 +84,16 @@ typedef unsigned char u_int8_t;
 #include <errno.h>
 #include <strings.h>
 #include <ctype.h>
+#include <stdint.h>
 #if defined(_QNX4)
 #include <sys/select.h>
 #include <unix.h>
 #endif
 
-#define _close close
+#define closeSocket close
 
 #ifdef SOLARIS
+#define u_int64_t uint64_t
 #define u_int32_t uint32_t
 #define u_int16_t uint16_t
 #define u_int8_t uint8_t
