@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up an AC3 audio elementary stream into frames
 // Implementation
 
@@ -237,7 +237,7 @@ void AC3AudioStreamParser
 		 unsigned char* ptr, unsigned size) {
   if (ourStreamCode == 0) return; // we assume that there's no stream code at the beginning of the data
 
-  if (size < 4) return; // shouldn't happen
+  if (size < 4) return;
   unsigned char streamCode = *ptr;
 
   if (streamCode == ourStreamCode) {
