@@ -62,8 +62,9 @@ Boolean AMRAudioFileSink::sourceIsCompatibleWithUs(MediaSource& source) {
   return source.isAMRAudioSource();
 }
 
-void AMRAudioFileSink::afterGettingFrame1(unsigned frameSize,
-					  struct timeval presentationTime) {
+void AMRAudioFileSink::afterGettingFrame(unsigned frameSize,
+					 unsigned numTruncatedBytes,
+					 struct timeval presentationTime) {
   AMRAudioSource* source = (AMRAudioSource*)fSource;
   if (source == NULL) return; // sanity check
 
@@ -96,5 +97,5 @@ void AMRAudioFileSink::afterGettingFrame1(unsigned frameSize,
   }
 
   // Call the parent class to complete the normal file write with the input data:
-  FileSink::afterGettingFrame1(frameSize, presentationTime);
+  FileSink::afterGettingFrame(frameSize, numTruncatedBytes, presentationTime);
 }
