@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A class used for digest authentication.
 // Implementation
 
@@ -30,9 +30,12 @@ Authenticator::Authenticator() {
   assign(NULL, NULL, NULL, NULL, False);
 }
 
+Authenticator::Authenticator(char const* username, char const* password, Boolean passwordIsMD5) {
+  assign(NULL, NULL, username, password, passwordIsMD5);
+}
+
 Authenticator::Authenticator(const Authenticator& orig) {
-  assign(orig.realm(), orig.nonce(), orig.username(), orig.password(),
-	 orig.fPasswordIsMD5);
+  assign(orig.realm(), orig.nonce(), orig.username(), orig.password(), orig.fPasswordIsMD5);
 }
 
 Authenticator& Authenticator::operator=(const Authenticator& rightSide) {
@@ -126,13 +129,13 @@ void Authenticator::reclaimDigestResponse(char const* responseStr) const {
 }
 
 void Authenticator::resetRealmAndNonce() {
-  delete fRealm; fRealm = NULL;
-  delete fNonce; fNonce = NULL;
+  delete[] fRealm; fRealm = NULL;
+  delete[] fNonce; fNonce = NULL;
 }
 
 void Authenticator::resetUsernameAndPassword() {
-  delete fUsername; fUsername = NULL;
-  delete fPassword; fPassword = NULL;
+  delete[] fUsername; fUsername = NULL;
+  delete[] fPassword; fPassword = NULL;
   fPasswordIsMD5 = False;
 }
 
@@ -141,17 +144,14 @@ void Authenticator::assignRealmAndNonce(char const* realm, char const* nonce) {
   fNonce = strDup(nonce);
 }
 
-void Authenticator
-::assignUsernameAndPassword(char const* username, char const* password,
-			    Boolean passwordIsMD5) {
+void Authenticator::assignUsernameAndPassword(char const* username, char const* password, Boolean passwordIsMD5) {
   fUsername = strDup(username);
   fPassword = strDup(password);
   fPasswordIsMD5 = passwordIsMD5;
 }
 
 void Authenticator::assign(char const* realm, char const* nonce,
-			   char const* username, char const* password,
-			   Boolean passwordIsMD5) {
+			   char const* username, char const* password, Boolean passwordIsMD5) {
   assignRealmAndNonce(realm, nonce);
   assignUsernameAndPassword(username, password, passwordIsMD5);
 }
