@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // File sinks
 // Implementation
 
@@ -81,11 +81,11 @@ Boolean FileSink::continuePlaying() {
 }
 
 void FileSink::afterGettingFrame(void* clientData, unsigned frameSize,
-				 unsigned /*numTruncatedBytes*/,
+				 unsigned numTruncatedBytes,
 				 struct timeval presentationTime,
 				 unsigned /*durationInMicroseconds*/) {
   FileSink* sink = (FileSink*)clientData;
-  sink->afterGettingFrame1(frameSize, presentationTime);
+  sink->afterGettingFrame(frameSize, numTruncatedBytes, presentationTime);
 }
 
 void FileSink::addData(unsigned char const* data, unsigned dataSize,
@@ -113,8 +113,15 @@ void FileSink::addData(unsigned char const* data, unsigned dataSize,
   }
 }
 
-void FileSink::afterGettingFrame1(unsigned frameSize,
-				  struct timeval presentationTime) {
+void FileSink::afterGettingFrame(unsigned frameSize,
+				 unsigned numTruncatedBytes,
+				 struct timeval presentationTime) {
+  if (numTruncatedBytes > 0) {
+    envir() << "FileSink::afterGettingFrame(): The input frame data was too large for our buffer size ("
+	    << fBufferSize << ").  "
+            << numTruncatedBytes << " bytes of trailing data was dropped!  Correct this by increasing the \"bufferSize\" parameter in the \"createNew()\" call to at least "
+            << fBufferSize + numTruncatedBytes << "\n";
+  }
   addData(fBuffer, frameSize, presentationTime);
 
   if (fOutFid == NULL || fflush(fOutFid) == EOF) {
