@@ -65,7 +65,11 @@ void H264VideoStreamDiscreteFramer
   // Check for a (likely) common error: NAL units that (erroneously) begin with a 0x00000001 or 0x000001 'start code'
   //     (Those start codes should only be in byte-stream data; not data that consists of discrete NAL units.)
   if (nal_unit_type == 0) {
-    envir() << "Warning: Invalid 'nal_unit_type': 0.  Does the NAL unit begin with a MPEG 'start code' by mistake?\n";
+    if (frameSize >= 4 && fTo[0] == 0 && fTo[1] == 0 && ((fTo[2] == 0 && fTo[3] == 1) || fTo[2] == 1)) {
+      envir() << "H264VideoStreamDiscreteFramer error: MPEG 'start code' seen in the input\n";
+    } else {
+      envir() << "Warning: Invalid 'nal_unit_type': 0\n";
+    }
   } else if (nal_unit_type == 7) { // Sequence parameter set (SPS)
     saveCopyOfSPS(fTo, frameSize);
   } else if (nal_unit_type == 8) { // Picture parameter set (PPS)
