@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // An abstract parser for MPEG video streams
 // Implementation
 
@@ -37,12 +37,6 @@ void MPEGVideoStreamParser::restoreSavedParserState() {
   fNumTruncatedBytes = fSavedNumTruncatedBytes;
 }
 
-void MPEGVideoStreamParser::setParseState() {
-  fSavedTo = fTo;
-  fSavedNumTruncatedBytes = fNumTruncatedBytes;
-  saveParserState();
-}
-
 void MPEGVideoStreamParser::registerReadInterest(unsigned char* to,
 						 unsigned maxSize) {
   fStartOfFrame = fTo = fSavedTo = to;
