@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that represents an existing
 // 'RTPSink', rather than one that creates new 'RTPSink's on demand.
 // Implementation
@@ -34,6 +34,30 @@ PassiveServerMediaSubsession
 ::PassiveServerMediaSubsession(RTPSink& rtpSink, RTCPInstance* rtcpInstance)
   : ServerMediaSubsession(rtpSink.envir()),
     fSDPLines(NULL), fRTPSink(rtpSink), fRTCPInstance(rtcpInstance) {
+  fClientRTCPSourceRecords = HashTable::create(ONE_WORD_HASH_KEYS);
+}
+
+class RTCPSourceRecord {
+public:
+  RTCPSourceRecord(netAddressBits addr, Port const& port)
+    : addr(addr), port(port) {
+  }
+
+  netAddressBits addr;
+  Port port;
+};
+
+PassiveServerMediaSubsession::~PassiveServerMediaSubsession() {
+  delete[] fSDPLines;
+
+  // Clean out the RTCPSourceRecord table:
+  while (1) {
+    RTCPSourceRecord* source = (RTCPSourceRecord*)(fClientRTCPSourceRecords->RemoveNext());
+    if (source == NULL) break;
+    delete source;
+  }
+
+  delete fClientRTCPSourceRecords;
 }
 
 char const*
@@ -42,7 +66,7 @@ PassiveServerMediaSubsession::sdpLines() {
     // Construct a set of SDP lines that describe this subsession:
     // Use the components from "rtpSink":
     Groupsock const& gs = fRTPSink.groupsockBeingUsed();
-    struct in_addr const& ipAddress = gs.groupAddress();
+    AddressString groupAddressStr(gs.groupAddress());
     unsigned short portNum = ntohs(gs.port().num());
     unsigned char ttl = gs.ttl();
     unsigned char rtpPayloadType = fRTPSink.rtpPayloadType();
@@ -54,8 +78,6 @@ PassiveServerMediaSubsession::sdpLines() {
     char const* auxSDPLine = fRTPSink.auxSDPLine();
     if (auxSDPLine == NULL) auxSDPLine = "";
 
-    char* const ipAddressStr = strDup(our_inet_ntoa(ipAddress));
-
     char const* const sdpFmt =
       "m=%s %d RTP/AVP %d\r\n"
       "c=IN IP4 %s/%d\r\n"
@@ -66,7 +88,7 @@ PassiveServerMediaSubsession::sdpLines() {
       "a=control:%s\r\n";
     unsigned sdpFmtSize = strlen(sdpFmt)
       + strlen(mediaType) + 5 /* max short len */ + 3 /* max char len */
-      + strlen(ipAddressStr) + 3 /* max char len */
+      + strlen(groupAddressStr.val()) + 3 /* max char len */
       + 20 /* max int len */
       + strlen(rtpmapLine)
       + strlen(rangeLine)
@@ -77,14 +99,14 @@ PassiveServerMediaSubsession::sdpLines() {
 	    mediaType, // m= <media>
 	    portNum, // m= <port>
 	    rtpPayloadType, // m= <fmt list>
-	    ipAddressStr, // c= <connection address>
+	    groupAddressStr.val(), // c= <connection address>
 	    ttl, // c= TTL
 	    estBitrate, // b=AS:<bandwidth>
 	    rtpmapLine, // a=rtpmap:... (if present)
 	    rangeLine, // a=range:... (if present)
 	    auxSDPLine, // optional extra SDP line
 	    trackId()); // a=control:<track-id>
-    delete[] ipAddressStr; delete[] (char*)rangeLine; delete[] rtpmapLine;
+    delete[] (char*)rangeLine; delete[] rtpmapLine;
 
     fSDPLines = strDup(sdpLines);
     delete[] sdpLines;
@@ -94,10 +116,10 @@ PassiveServerMediaSubsession::sdpLines() {
 }
 
 void PassiveServerMediaSubsession
-::getStreamParameters(unsigned /*clientSessionId*/,
-		      netAddressBits /*clientAddress*/,
+::getStreamParameters(unsigned clientSessionId,
+		      netAddressBits clientAddress,
 		      Port const& /*clientRTPPort*/,
-		      Port const& /*clientRTCPPort*/,
+		      Port const& clientRTCPPort,
 		      int /*tcpSocketNum*/,
 		      unsigned char /*rtpChannelId*/,
 		      unsigned char /*rtcpChannelId*/,
@@ -126,19 +148,20 @@ void PassiveServerMediaSubsession
     serverRTCPPort = rtcpGS->port();
   }
   streamToken = NULL; // not used
+
+  // Make a record of this client's source - for RTCP RR handling:
+  RTCPSourceRecord* source = new RTCPSourceRecord(clientAddress, clientRTCPPort);
+  fClientRTCPSourceRecords->Add((char const*)clientSessionId, source);
 }
 
-void PassiveServerMediaSubsession::startStream(unsigned /*clientSessionId*/,
+void PassiveServerMediaSubsession::startStream(unsigned clientSessionId,
 					       void* /*streamToken*/,
-					       TaskFunc* /*rtcpRRHandler*/,
-					       void* /*rtcpRRHandlerClientData*/,
+					       TaskFunc* rtcpRRHandler,
+					       void* rtcpRRHandlerClientData,
 					       unsigned short& rtpSeqNum,
 					       unsigned& rtpTimestamp,
 					       ServerRequestAlternativeByteHandler* /*serverRequestAlternativeByteHandler*/,
 					       void* /*serverRequestAlternativeByteHandlerClientData*/) {
-  // Note: We don't set a RTCP RR handler, because (i) we're called potentially
-  // many times on the same "RTCPInstance", and (ii) the "RTCPInstance" remains
-  // in existence after "deleteStream()" is called.
   rtpSeqNum = fRTPSink.currentSeqNo();
   rtpTimestamp = fRTPSink.presetNextTimestamp();
 
@@ -148,8 +171,26 @@ void PassiveServerMediaSubsession::startStream(unsigned /*clientSessionId*/,
   unsigned rtpBufSize = streamBitrate * 25 / 2; // 1 kbps * 0.1 s = 12.5 bytes
   if (rtpBufSize < 50 * 1024) rtpBufSize = 50 * 1024;
   increaseSendBufferTo(envir(), fRTPSink.groupsockBeingUsed().socketNum(), rtpBufSize);
+
+  // Set up the handler for incoming RTCP "RR" packets from this client:
+  if (fRTCPInstance != NULL) {
+    RTCPSourceRecord* source = (RTCPSourceRecord*)(fClientRTCPSourceRecords->Lookup((char const*)clientSessionId));
+    if (source != NULL) {
+      fRTCPInstance->setSpecificRRHandler(source->addr, source->port,
+					  rtcpRRHandler, rtcpRRHandlerClientData);
+    }
+  }
 }
 
-PassiveServerMediaSubsession::~PassiveServerMediaSubsession() {
-  delete[] fSDPLines;
+void PassiveServerMediaSubsession::deleteStream(unsigned clientSessionId, void*& /*streamToken*/) {
+  // Lookup and remove the 'RTCPSourceRecord' for this client.  Also turn off RTCP "RR" handling:
+  RTCPSourceRecord* source = (RTCPSourceRecord*)(fClientRTCPSourceRecords->Lookup((char const*)clientSessionId));
+  if (source != NULL) {
+    if (fRTCPInstance != NULL) {
+      fRTCPInstance->unsetSpecificRRHandler(source->addr, source->port);
+    }
+
+    fClientRTCPSourceRecords->Remove((char const*)clientSessionId);
+    delete source;
+  }
 }
