@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // Abstract class for parsing a byte stream
 // Implementation
 
@@ -112,6 +112,10 @@ unsigned StreamParser::getBits(unsigned numBits) {
   }
 }
 
+unsigned StreamParser::bankSize() const {
+  return BANK_SIZE;
+}
+
 #define NO_MORE_BUFFERED_INPUT 1
 
 void StreamParser::ensureValidBytes1(unsigned numBytesNeeded) {
@@ -141,7 +145,7 @@ void StreamParser::ensureValidBytes1(unsigned numBytesNeeded) {
     // If this happens, it means that we have too much saved parser state.
     // To fix this, increase BANK_SIZE as appropriate.
     fInputSource->envir() << "StreamParser internal error ("
-			  << fCurParserIndex << "+ "
+			  << fCurParserIndex << " + "
 			  << numBytesNeeded << " > "
 			  << BANK_SIZE << ")\n";
     fInputSource->envir().internalError();
@@ -195,7 +199,7 @@ void StreamParser::onInputClosure(void* clientData) {
 void StreamParser::onInputClosure1() {
   if (!fHaveSeenEOF) {
     // We're hitting EOF for the first time.  Set our 'EOF' flag, and continue parsing, as if we'd just read 0 bytes of data.
-    // This allows the parser to re-parse any remaining unparsed code (perhaps while testing for EOF at the end):
+    // This allows the parser to re-parse any remaining unparsed data (perhaps while testing for EOF at the end):
     fHaveSeenEOF = True;
     afterGettingBytes1(0, fLastSeenPresentationTime);
   } else {
