@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A sink representing a TCP output stream
 // Implementation
 
@@ -89,10 +89,15 @@ void TCPStreamSink::socketWritableHandler1() {
 void TCPStreamSink::afterGettingFrame(void* clientData, unsigned frameSize, unsigned numTruncatedBytes,
 				struct timeval /*presentationTime*/, unsigned /*durationInMicroseconds*/) {
   TCPStreamSink* sink = (TCPStreamSink*)clientData;
-  sink->afterGettingFrame1(frameSize, numTruncatedBytes);
+  sink->afterGettingFrame(frameSize, numTruncatedBytes);
 }
 
-void TCPStreamSink::afterGettingFrame1(unsigned frameSize, unsigned numTruncatedBytes) {
+void TCPStreamSink::afterGettingFrame(unsigned frameSize, unsigned numTruncatedBytes) {
+  if (numTruncatedBytes > 0) {
+    envir() << "TCPStreamSink::afterGettingFrame(): The input frame data was too large for our buffer.  "
+	    << numTruncatedBytes
+	    << " bytes of trailing data was dropped!  Correct this by increasing the definition of \"TCP_STREAM_SINK_BUFFER_SIZE\" in \"include/TCPStreamSink.hh\".\n";
+  }
   fUnwrittenBytesEnd += frameSize;
   processBuffer();
 }
