@@ -51,6 +51,8 @@ unsigned WAVAudioFileSource::numPCMBytes() const {
 }
 
 void WAVAudioFileSource::setScaleFactor(int scale) {
+  if (!fFidIsSeekable) return; // we can't do 'trick play' operations on non-seekable files
+
   fScaleFactor = scale;
 
   if (fScaleFactor < 0 && TellFile64(fFid) > 0) {
@@ -104,8 +106,8 @@ static Boolean skipBytes(FILE* fid, int num) {
 
 WAVAudioFileSource::WAVAudioFileSource(UsageEnvironment& env, FILE* fid)
   : AudioInputDevice(env, 0, 0, 0, 0)/* set the real parameters later */,
-    fFid(fid), fLastPlayTime(0), fWAVHeaderSize(0), fFileSize(0), fScaleFactor(1),
-    fLimitNumBytesToStream(False), fNumBytesToStream(0), fAudioFormat(WA_UNKNOWN) {
+    fFid(fid), fFidIsSeekable(False), fLastPlayTime(0), fHaveStartedReading(False), fWAVHeaderSize(0), fFileSize(0),
+    fScaleFactor(1), fLimitNumBytesToStream(False), fNumBytesToStream(0), fAudioFormat(WA_UNKNOWN) {
   // Check the WAV file header for validity.
   // Note: The following web pages contain info about the WAV format:
   // http://www.ringthis.com/dev/wave_format.htm
@@ -192,20 +194,66 @@ WAVAudioFileSource::WAVAudioFileSource(UsageEnvironment& env, FILE* fid)
   unsigned desiredSamplesPerFrame = (unsigned)(0.02*fSamplingFrequency);
   unsigned samplesPerFrame = desiredSamplesPerFrame < maxSamplesPerFrame ? desiredSamplesPerFrame : maxSamplesPerFrame;
   fPreferredFrameSize = (samplesPerFrame*fNumChannels*fBitsPerSample)/8;
+
+  fFidIsSeekable = FileIsSeekable(fFid);
+#ifndef READ_FROM_FILES_SYNCHRONOUSLY
+  // Now that we've finished reading the WAV header, all future reads (of audio samples) from the file will be asynchronous:
+  makeSocketNonBlocking(fileno(fFid));
+#endif
 }
 
 WAVAudioFileSource::~WAVAudioFileSource() {
+  if (fFid == NULL) return;
+
+#ifndef READ_FROM_FILES_SYNCHRONOUSLY
+  envir().taskScheduler().turnOffBackgroundReadHandling(fileno(fFid));
+#endif
+
   CloseInputFile(fFid);
 }
 
-// Note: We should change the following to use asynchronous file reading, #####
-// as we now do with ByteStreamFileSource. #####
 void WAVAudioFileSource::doGetNextFrame() {
   if (feof(fFid) || ferror(fFid) || (fLimitNumBytesToStream && fNumBytesToStream == 0)) {
     handleClosure(this);
     return;
   }
 
+  fFrameSize = 0; // until it's set later
+#ifdef READ_FROM_FILES_SYNCHRONOUSLY
+  doReadFromFile();
+#else
+  if (!fHaveStartedReading) {
+    // Await readable data from the file:
+    envir().taskScheduler().turnOnBackgroundReadHandling(fileno(fFid),
+							 (TaskScheduler::BackgroundHandlerProc*)&fileReadableHandler, this);
+    fHaveStartedReading = True;
+  }
+#endif
+}
+
+void WAVAudioFileSource::doStopGettingFrames() {
+#ifndef READ_FROM_FILES_SYNCHRONOUSLY
+  envir().taskScheduler().turnOffBackgroundReadHandling(fileno(fFid));
+  fHaveStartedReading = False;
+#endif
+}
+
+void WAVAudioFileSource::fileReadableHandler(WAVAudioFileSource* source, int /*mask*/) {
+  if (!source->isCurrentlyAwaitingData()) {
+    source->doStopGettingFrames(); // we're not ready for the data yet
+    return;
+  }
+  source->doReadFromFile();
+}
+
+static Boolean const readFromFilesSynchronously
+#ifdef READ_FROM_FILES_SYNCHRONOUSLY
+= True;
+#else
+= False;
+#endif
+
+void WAVAudioFileSource::doReadFromFile() {
   // Try to read as many bytes as will fit in the buffer provided (or "fPreferredFrameSize" if less)
   if (fLimitNumBytesToStream && fNumBytesToStream < fMaxSize) {
     fMaxSize = fNumBytesToStream;
@@ -215,24 +263,36 @@ void WAVAudioFileSource::doGetNextFrame() {
   }
   unsigned bytesPerSample = (fNumChannels*fBitsPerSample)/8;
   if (bytesPerSample == 0) bytesPerSample = 1; // because we can't read less than a byte at a time
-  unsigned bytesToRead = fMaxSize - fMaxSize%bytesPerSample;
-  if (fScaleFactor == 1) {
-    // Common case - read samples in bulk:
-    fFrameSize = fread(fTo, 1, bytesToRead, fFid);
-    fNumBytesToStream -= fFrameSize;
-  } else {
-    // We read every 'fScaleFactor'th sample:
-    fFrameSize = 0;
-    while (bytesToRead > 0) {
-      size_t bytesRead = fread(fTo, 1, bytesPerSample, fFid);
-      if (bytesRead <= 0) break;
-      fTo += bytesRead;
-      fFrameSize += bytesRead;
-      fNumBytesToStream -= bytesRead;
-      bytesToRead -= bytesRead;
 
-      // Seek to the appropriate place for the next sample:
+  // For 'trick play', read one sample at a time; otherwise (normal case) read samples in bulk:
+  unsigned bytesToRead = fScaleFactor == 1 ? fMaxSize - fMaxSize%bytesPerSample : bytesPerSample;
+  unsigned numBytesRead;
+  while (1) { // loop for 'trick play' only
+    if (readFromFilesSynchronously || fFidIsSeekable) {
+      numBytesRead = fread(fTo, 1, bytesToRead, fFid);
+   } else {
+      // For non-seekable files (e.g., pipes), call "read()" rather than "fread()", to ensure that the read doesn't block:
+      numBytesRead = read(fileno(fFid), fTo, bytesToRead);
+    }
+    if (numBytesRead == 0) {
+     handleClosure(this);
+      return;
+    }
+    fFrameSize += numBytesRead;
+    fTo += numBytesRead;
+    fMaxSize -= numBytesRead;
+    fNumBytesToStream -= numBytesRead;
+
+    // If we did an asynchronous read, and didn't read an integral number of samples, then we need to wait for another read:
+    if (!readFromFilesSynchronously && fFrameSize%bytesPerSample > 0) return;
+    
+    // If we're doing 'trick play', then seek to the appropriate place for reading the next sample,
+    // and keep reading until we fill the provided buffer:
+    if (fScaleFactor != 1) {
       SeekFile64(fFid, (fScaleFactor-1)*bytesPerSample, SEEK_CUR);
+      if (fMaxSize < bytesPerSample) break;
+    } else {
+      break; // from the loop (normal case)
     }
   }
 
@@ -251,19 +311,15 @@ void WAVAudioFileSource::doGetNextFrame() {
   fDurationInMicroseconds = fLastPlayTime
     = (unsigned)((fPlayTimePerSample*fFrameSize)/bytesPerSample);
 
-  // Switch to another task, and inform the reader that he has data:
-#if defined(__WIN32__) || defined(_WIN32)
-  // HACK: One of our applications that uses this source uses an
-  // implementation of scheduleDelayedTask() that performs very badly
-  // (chewing up lots of CPU time, apparently polling) on Windows.
-  // Until this is fixed, we just call our "afterGetting()" function
-  // directly.  This avoids infinite recursion, as long as our sink
-  // is discontinuous, which is the case for the RTP sink that
-  // this application uses. #####
-  afterGetting(this);
-#else
+  // Inform the reader that he has data:
+#ifdef READ_FROM_FILES_SYNCHRONOUSLY
+  // To avoid possible infinite recursion, we need to return to the event loop to do this:
   nextTask() = envir().taskScheduler().scheduleDelayedTask(0,
-			(TaskFunc*)FramedSource::afterGetting, this);
+                                (TaskFunc*)FramedSource::afterGetting, this);
+#else
+  // Because the file read was done from the event loop, we can call the
+  // 'after getting' function directly, without risk of infinite recursion:
+  FramedSource::afterGetting(this);
 #endif
 }
 
